"""
- :class:`nbfigtulz.config.config` as ``nbfigtulz.config``
- :class:`nbfigtulz.figtools.FigContext` as ``nbfigtulz.FigContext``
- :class:`nbfigtulz.figtools.with_context` as ``nbfigtulz.with_context``
- :class:`nbfigtulz.figtools.save_fig` as ``nbfigtulz.save_fig``
- :class:`nbfigtulz.figtools.img_grid` as ``nbfigtulz.img_grid``
- :class:`nbfigtulz.figtools.Size` as ``nbfigtulz.fig_size``
"""

from .config import config
from .figtools import FigContext, with_context, save_fig, img_grid
from .figtools import Size as fig_size